
	LIST P=18F4320, F=INHX32 ;directive to define processor and file format
	#include <P18F4320.INC>	 ;processor specific variable definitions
	
	CONFIG	OSC=HS, WDT=OFF, MCLRE=ON, PWRT=OFF, BOR=OFF, LVP=OFF
	
VIDBUFSIZE		EQU		0x40
	
	CBLOCK	0x00
	VIDEO_BUFFER : VIDBUFSIZE		; NOTE: 45 needed for line
	ENDC
	
	CBLOCK	0x40
	ByteCount
	STAT_TMP
	ENDC
	
LOAD_TBLPTR	MACRO	ptr
	MOVLW	(ptr >> 16) & 0xFF
	MOVWF	TBLPTRU
	MOVLW	(ptr >> 8) & 0xFF
	MOVWF	TBLPTRH
	MOVLW	ptr & 0xFF
	MOVWF	TBLPTRL
	ENDM

DIE	MACRO
	NOP
	GOTO	$-2
	ENDM

	ORG 0x0000
RESET_VECTOR
	GOTO	Main		;go to start of main code

; video ISR
	ORG	0x0008
HI_INT_VECTOR
	MOVFF	PORTD, POSTINC0		; read in video byte
	BCF		INTCON, INT0IF
	RETFIE	FAST

; UART ISR
	ORG 0x0018
LOW_INT_VECTOR
	MOVFF	STATUS, STAT_TMP
		
	MOVFF	POSTINC1, TXREG		; interrupt flag cleared on TXREG write

	MOVFF	STAT_TMP, STATUS
	RETFIE

Main:	
	CALL 	Init_Ports

	CALL	UART_Init
	CALL	I2C_Master_Init
	
	CALL	Strobe_Resets
	
	CALL	Init_Interrupts
	
	MOVLW	'Z'					; WTF? Why does not having this break the ISR?
	
	; configure video chip
	LOAD_TBLPTR TVP_SHR
	CALL	I2C_Send
	LOAD_TBLPTR TVP_CHSEL
	CALL	I2C_Send
	LOAD_TBLPTR TVP_MISC
	CALL	I2C_Send
	LOAD_TBLPTR TVP_MSK
	CALL	I2C_Send
	LOAD_TBLPTR TVP_FMT
	CALL	I2C_Send
	LOAD_TBLPTR TVP_CHRM1
	CALL	I2C_Send
	LOAD_TBLPTR TVP_CHRM2
	CALL	I2C_Send
	
	; set up indirect addressing
	LFSR	FSR0, VIDEO_BUFFER
	LFSR	FSR1, VIDEO_BUFFER
	
	; turn on global interrupts before TXIF so we start filling
	; buffer before we start sending
	BSF		INTCON, GIE
	BSF		PIE1, TXIE
	
main_loop:
	MOVLW	VIDBUFSIZE
	CPFSEQ	FSR0L
	GOTO	main_loop
	
	BCF		INTCON, INT0IE
	
	MOVLW	VIDBUFSIZE
	CPFSEQ	FSR1L
	GOTO	$-2
	
	BCF		PIE1, TXIE

	; set up indirect addressing
;	LFSR	FSR0, VIDEO_BUFFER
;	LFSR	FSR1, VIDEO_BUFFER
	
	; turn on INT0IF then TXIF
;	BSF		INTCON, INTOIE
;	BSF		PIE1, TXIE
	
;	GOTO	main_loop
	
	DIE
	
Init_Ports:
	; configure PORTE as outputs, using as reset strobes	
	; set all analog channels to digital I/O
	MOVLW	0x0F
	MOVWF	ADCON1
	
	; set PORTE as output and clear
	CLRF	TRISE
	CLRF	PORTE
	
	; set PORTD as input
	SETF	TRISD
	
	RETURN
	
Init_Interrupts:
	; enable interrupts - global, peripheral, INT0
	; GIE, PEIE, -, INT0IE, -, -, -, -
	MOVLW	b'01010000'
	MOVWF	INTCON
	
	; set all INTx pins to rising edge
	MOVLW	b'01110000'
	MOVWF	INTCON2
	
	; set UART TXIF to low priority
	BCF		IPR1, TXIP
	
	; enable interrupt priorities
	BSF		RCON, IPEN
	RETURN
	
Strobe_Resets:
	; RE0 : bluetooth reset - active high
	; RE1 : TVP reset - active low
	MOVLW	b'00000010'
	MOVWF	PORTE
	
	MOVLW	0x03
	XORWF	PORTE, F
	NOP
	NOP
	XORWF	PORTE, F
	
	; yay! stuff works when you reset it!	
	RETURN
	
UART_Init:
	; configure ports NOTE: could cause problems if set TRISC after fn call
	BCF		TRISC, RC6	; Tx is output
	BSF		TRISC, RC7	; Rx is input
	
	; initialize BRG for UART
	BSF		TXSTA, BRGH	; use high speed UART
	MOVLW	d'21'		; osc=20MHz => 96.2Kbps
	MOVWF	SPBRG
	
	; initialize UART
	BCF		TXSTA, SYNC		; async
	BSF		TXSTA, TXEN		; enable transmit
	BSF		RCSTA, SPEN		; set RC7/RC6 as Rx/Tx
	BSF		RCSTA, CREN
	
	RETURN
	
UART_Send:
	TBLRD*+
	MOVFF	TABLAT, ByteCount
	
UART_Send_Byte:
	TBLRD*+
	MOVFF	TABLAT, TXREG
	NOP
	NOP
	BTFSS	PIR1, TXIF		; wait for transmission of byte
	GOTO	$-2
	
	DECFSZ	ByteCount
	GOTO	UART_Send_Byte
		
	RETURN

I2C_Master_Init:
	; configure I2C ports as input (SDA, SCL)
	BSF		TRISC, RB3
	BSF		TRISC, RB4
	
	; baud rate : 100 KHz
	MOVLW	0x63
	MOVWF	SSPADD, 0
	
	; enable slew control
	MOVLW	0x00
	MOVWF	SSPSTAT, 0
	
	; enable I2C master serial mode
	MOVLW	b'00111000'
	MOVWF	SSPCON1, 0
	
	RETURN
	
I2C_Send:
	TBLRD*+
	MOVFF	TABLAT, ByteCount
	
	; start condition
	BSF		SSPCON2, SEN, 0
	; wait
	BTFSC	SSPCON2, SEN
	GOTO	$-2

I2C_Send_Byte:
	TBLRD*+
	MOVFF	TABLAT, SSPBUF
	
	; wait
	BTFSC	SSPSTAT, R_W
	GOTO	$-2
		
	; check for ACK bit (ACKSTAT = 0)
	BTFSC	SSPCON2, ACKSTAT
	GOTO	I2C_Quit_Send
	
	DECFSZ	ByteCount
	GOTO	I2C_Send_Byte

I2C_Quit_Send
	; do stop condition
	BSF		SSPCON2, PEN
	BTFSC	SSPCON2, PEN, 0
	GOTO	$-2
	
	MOVF	ByteCount, W
	
	RETURN

; structure: length, addrs, regs, vals
	ORG	0x0700
TVP_SHR		db	0x03, 0xBA, 0x0F, 0x02
TVP_CHSEL	db	0x03, 0xBA, 0x01, 0x15
;TVP_MISC	db	0x03, 0xBA, 0x03, 0x6F
TVP_MISC	db	0x03, 0xBA, 0x03, 0xEF
TVP_MSK		db	0x03, 0xBA, 0x04, 0x00
TVP_FMT		db	0x03, 0xBA, 0x0D, 0x40
TVP_CHRM1	db	0x03, 0xBA, 0x1A, 0x0C
TVP_CHRM2	db	0x03, 0xBA, 0x1B, 0x54

HELLO_WORLD		db	0xD, "HELLO WORLD! "
	END
